package com.esri.gwt.client;

import com.esri.gwt.client.dojo.Deferred;
import com.google.gwt.core.client.JavaScriptObject;

/**
 * This class provides the framework and helper methods required to implement a solution for managing user credentials for the following resources:
 *   ArcGIS Server resources secured using token-based authentication. Note that only ArcGIS Server versions 10 SP 1 and greater are supported.
 *   Secured ArcGIS.com resources (i.e. web maps).
 *   
 *   This class is not typically used by itself and does not include a user interface to obtain user input. The esri.IdentityManager class provides a complete 
 *   out-of-the-box implementation.
 * 
 * @author Justin Merz
 */
public class IdentityManagerBase extends JavaScriptObject {
	
	protected IdentityManagerBase() {}
	
	/**
	 * The suggested lifetime of the token in minutes. Default is 60 minutes.
	 * 
	 * @return int
	 */
	public final native int getTokenValidity() /*-{
		return this.tokenValidity;
	}-*/;

	/**
	 * Returns the credential for the resource identified by the specified url. Optionally you can provide a userId to find credentials for a specific user.
	 * 
	 * @param url - The url to a server.
	 * @param userId - The userId for which you want to obtain credentials.
	 * @return Credential
	 */
	public final native Credential findCredential(String url, String userId) /*-{
		return this.findCredential(url, userId);
	}-*/;
	
	/**
	 * Returns information about the server that is hosting the specified url.
	 * 
	 * @param url - The url to a server.
	 * @return ServerInfo
	 */
	public final native ServerInfo findServerInfo(String url) /*-{
		return this.findServerInfo(url);
	}-*/;
	
	/**
	 * Returns an object containing a token and its expiration time. You need to provide the ServerInfo object that contains token service URL and a 
	 * user info object containing username and password. This is a helper method typically called by sub-classes to generate tokens.
	 * 
	 * @param serverInfo - A ServerInfo object that contains a token service URL.
	 * @param userInfo - A user info object containing a user name and password.
	 * @return Deferred
	 */
	public final native Deferred generateToken(ServerInfo serverInfo, JavaScriptObject userInfo) /*-{
		return this.generateToken(serverInfo, userInfo);
	}-*/;
	
	/**
	 * Returns a Credential object that can be used to access the secured resource identified by the input url. If required the user will be challenged 
	 * for a username and password which is used to generate a token. Note: The IdentityManager sets up a timer to update the Credential object with 
	 * a new token prior to the expiration time. This method is typically called by esri.request when a request fails due to an "invalid credentials" error.
	 * 
	 * @param url - The url for the secure resource.
	 * @param retry - Determines if the method should make additional attempts to get the credentials after a failure.
	 * @return Deferred
	 */
	public final native Deferred getCredential(String url, boolean retry) /*-{
		return this.getCredential(url, retry);
	}-*/;
	
	/**
	 * Returns true if the identity manager is busy accepting user input, i.e. the user has invoked signIn and is waiting for a response.
	 * 
	 * @return boolean
	 */
	public final native boolean isBusy() /*-{
		return this.isBursy();
	}-*/;
	
	/**
	 * Sub-classes must implement this method to create and manager the user interface that is used to obtain a username and password 
	 * from the end-user. It should perform the following tasks:
	 *     Challenge the user for a username and password.
	 *     Generate a token and return it to the caller via Deferred object.
	 * 
	 * @param url - Url for the secure resource.
	 * @param serverInfo - A ServerInfo object that contains the token service url.
	 * @return Deferred
	 */
	public final native Deferred signIn(String url, ServerInfo serverInfo)/*-{
		return this.signIn(url, serverInfo);
	}-*/;
	
}
