package com.esri.gwt.client;


import java.util.List;

import com.esri.gwt.client.control.Control;
import com.esri.gwt.client.control.ControlHandler;
import com.esri.gwt.client.dijits.InfoWindow;
import com.esri.gwt.client.dijits.TimeSlider;
import com.esri.gwt.client.event.ClickHandler;
import com.esri.gwt.client.event.DblClickHandler;
import com.esri.gwt.client.event.MapExtentChangeHandler;
import com.esri.gwt.client.event.MapKeyDownHandler;
import com.esri.gwt.client.event.MapKeyUpHandler;
import com.esri.gwt.client.event.MapLayerAddResultHandler;
import com.esri.gwt.client.event.MapLayersAddResultHandler;
import com.esri.gwt.client.event.MapLoadHandler;
import com.esri.gwt.client.event.MapMouseDragEndHandler;
import com.esri.gwt.client.event.MapMouseDragHandler;
import com.esri.gwt.client.event.MapLayerAddHandler;
import com.esri.gwt.client.event.MapLayerRemoveHandler;
import com.esri.gwt.client.event.MapLayerReorderHandler;
import com.esri.gwt.client.event.MapLayersRemovedHandler;
import com.esri.gwt.client.event.MapLayersReorderedHandler;
import com.esri.gwt.client.event.MapMouseDragStartHandler;
import com.esri.gwt.client.event.MapPanEndHandler;
import com.esri.gwt.client.event.MapPanHandler;
import com.esri.gwt.client.event.MapPanStartHandler;
import com.esri.gwt.client.event.MapRepositionHandler;
import com.esri.gwt.client.event.MapResizeHandler;
import com.esri.gwt.client.event.MapUnloadHandler;
import com.esri.gwt.client.event.MapZoomEndHandler;
import com.esri.gwt.client.event.MapZoomHandler;
import com.esri.gwt.client.event.MapZoomStartHandler;
import com.esri.gwt.client.event.MouseDownHandler;
import com.esri.gwt.client.event.MouseMoveHandler;
import com.esri.gwt.client.event.MouseOutHandler;
import com.esri.gwt.client.event.MouseOverHandler;
import com.esri.gwt.client.event.MouseUpHandler;
import com.esri.gwt.client.event.MouseWheelHandler;
import com.esri.gwt.client.event.TimeExtentChangeHandler;
import com.esri.gwt.client.geometry.Extent;
import com.esri.gwt.client.geometry.Point;
import com.esri.gwt.client.layers.GraphicsLayer;
import com.esri.gwt.client.layers.LOD;
import com.esri.gwt.client.layers.Layer;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.core.client.JsArrayString;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.Style.Position;
import com.google.gwt.event.logical.shared.AttachEvent;
import com.google.gwt.event.logical.shared.AttachEvent.Handler;
import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.Widget;


/**
 * Basic Map class with all the navigation functionalities.
 * 
 * The Map.onLoad event is fired after the first layer has been added to the map. 
 * At this point, the map is fully functional. A Map.onLoad handler function should 
 * be used to add content and change map behaviors. 
 * 
 * @author Justin Merz
 */
public class MapWidget extends Composite {

	// esri.map object
	private JavaScriptObject map = null;

	// counter used for creating unique map id's
	private static int nextMapId = 0;
	
	// Gwt added Control Layer
	private AbsolutePanel controlLayer = new AbsolutePanel();
	private ControlHandler controlHandler;
	
	// Div that esri.map will be constructed with
	private final FlowPanel mapContainer = new FlowPanel();
	
	// Initial options supplied to contructor
	private Options mapOptions = null;
	private MapLoadHandler onLoadHandler = null;
	private Layer initLayer = null;
	
	/**
	 * Creates a new map. 
	 * 
	 * @param initializationLayer - Layer map will be initialized to.  After the map is added to the dom, this will be
	 * the first layer added to the map.  Once the map is initialized the onLoad handler will be called.  Can be null.
	 * @param onLoad - This is the map load handler to be fired once the map as been added to the dom and the
	 * first layer has been initialized.  Can be null. 
	 */
	public MapWidget(Layer initializationLayer, MapLoadHandler onLoad) {
	    	assertLoaded();
	    	
		   mapContainer.getElement().setId("map_"+nextMapId);
		   mapContainer.getElement().getStyle().setPosition(Position.RELATIVE);
		   
		   initWidget(mapContainer);
		   
	    	onLoadHandler = onLoad;
	    	initLayer = initializationLayer;
		   
		   addAttachHandler(new Handler(){
				@Override
				public void onAttachOrDetach(AttachEvent event) {
					if( map == null ) {
						map = create(getElement());
						
						// inject controls layer
						initControls();
						
						if( initLayer != null ) addLayer(initLayer);
						
						if( onLoadHandler != null ) addLoadHandler(onLoadHandler);
						
					}
				}
		   });

		    nextMapId++;
	}

	/**
	 * Creates a new Map.
	 * 
	 *  @param initializationLayer - Layer map will be initialized to.  After the map is added to the dom, this will be
	 *  the first layer added to the map.  Once the map is initialized the onLoad handler will be called.
	 *  @param onLoad - This is the map load handler to be fired once the map as been added to the dom and the
	 *  first layer has been initialized.  Can be null. 
	 * @param options - Optional parameters for the map.
	 */
	public MapWidget(Layer initializationLayer, MapLoadHandler onLoad, Options options) {
	    assertLoaded();
	    
	   mapContainer.getElement().setId("map_"+nextMapId);
	   mapContainer.getElement().getStyle().setPosition(Position.RELATIVE);
	   
	   initWidget(mapContainer);
	   
	   mapOptions = options;
	   onLoadHandler = onLoad;
	   initLayer = initializationLayer;

	   addAttachHandler(new Handler(){
			@Override
			public void onAttachOrDetach(AttachEvent event) {			
				if( map == null ) {				
					map = create(getElement(), mapOptions);
					
					// inject controls layer
					initControls();
					
					if( initLayer != null ) addLayer(initLayer);
					
					if( onLoadHandler != null ) addLoadHandler(onLoadHandler);	
					
				}
			}
	   });

	    nextMapId++;
	}
	
	private void initControls() {
		controlLayer.getElement().getStyle().setZIndex(1000);
		mapContainer.add(controlLayer);
		controlHandler = new ControlHandler(this, controlLayer);
	}
	
	/**
	 * Add new control to the Control Layer.  
	 * 
	 * The control layer is not apart of the ESRI library, rather an
	 * added component in the GWT library. 
	 * 
	 * @param control - Control to be added to the map
	 */
	public void addControl(Control control){
		controlHandler.addControl(control);
	}
	
	/**
	 * Remove control from the Control Layer.
	 * 
	 * The control layer is not apart of the ESRI library, rather an
	 * added component in the GWT library. 
	 * 
	 * @param control - Control to be removed
	 */
	public void removeControl(Control control){
		controlHandler.removeControl(control);
	}
	
	/**
	 * Make sure the $wnd.esri library has been loaded.  Alert error
	 * message and return false if it has not been loaded.  Othewise
	 * return true.
	 * 
	 * @return boolean
	 */
	public static final native boolean assertLoaded() /*-{
		if( !$wnd.esri ) {
			$wnd.alert("ESRI API is not loaded!");
			return false;
		}
		if( $wnd.esri.Map ) return true;
		$wnd.alert("ESRI API is not loaded!");
		return false;
	}-*/;
	
	/**
	 * Return the actual ersi.map object that has been created
	 * 
	 * @return JavaScriptObject
	 */
	public JavaScriptObject getMapObject() {
		return this.map;
	}
	
	/**
	 * Updates the MapWidget's size.  If the map has been loaded, resize() will be called automatically.  If the map hasn't loaded (attached
	 * to the DOM with at least one layer loaded) you need to call resize() once the map is loaded or bad things will happen.
	 * You can add an event listener to onMapLoad to know when the map is loaded.
	 * 
	 * @param width - width MapWidget should be set to
	 * @param height - height MapWidget should be set to
	 */
	@Override
	public void setSize(String width, String height){
		super.setSize(width, height);
		if( isLoaded() ) resize();
	}
	
	/**
	 * Updates the MapWidget's width.  If the map has been loaded, resize() will be called automatically.  If the map hasn't loaded (attached
	 * to the DOM with at least one layer loaded) you need to call resize() once the map is loaded or bad things will happen.
	 * You can add an event listener to onMapLoad to know when the map is loaded.
	 * 
	 * @param width - width MapWidget should be set to
	 */
	@Override
	public void setWidth(String width){
		super.setWidth(width);
		if( isLoaded() ) resize();
	}
	
	/**
	 * Updates the MapWidget's height.  If the map has been loaded, resize() will be called automatically.  If the map hasn't loaded (attached
	 * to the DOM with at least one layer loaded) you need to call resize() once the map is loaded or bad things will happen.
	 * You can add an event listener to onMapLoad to know when the map is loaded.
	 * 
	 * @param height - height MapWidget should be set to
	 */
	@Override
	public void setHeight(String height){
		super.setHeight(height);
		if( isLoaded() ) resize();
	}
	
	/* 
	 * JSNI Map Contructors 
	 * /
	
	/*
	 * Create esri.map object using DOM element
	 */
	private native JavaScriptObject create(Element domElement) /*-{
		var map = new $wnd.esri.Map(domElement);
		map["__gwt_MapWidget"] = this;
		return map;
	}-*/;
	
	/*
	 * Create esri.map object for IE using id of Div element and options
	 */
	private native JavaScriptObject create(Element domElement, Options options) /*-{
		var map = new $wnd.esri.Map(domElement, options);
		map["__gwt_MapWidget"] = this;
		return map;
	}-*/;
	
	public void addInfoWindow(Widget panel) {
		mapContainer.add(panel);
	}
	
	/*
	 * Map Properties
	 */
	/**
	 * The current extent of the map in map units. 
	 * 
	 * @return boolean
	 */
	public native Extent getExtent() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.extent;
	}-*/;
	
	/**
	 * Add and remove graphics from the GraphicsLayer.
	 * 
	 * @return GraphicsLayer
	 */
	public native GraphicsLayer getGraphics() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.graphics;
	}-*/;

	/**
	 * An array of the current GraphicsLayers in the map.
	 * 
	 * @return JsArrayString
	 */
	public native JsArrayString getGraphicsLayerIds() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.graphicsLayerIds;
	}-*/;
	
	/**
	 * Current height of the map in screen pixels.
	 * 
	 * @return int
	 */
	public native int getHeight() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.height;
	}-*/;
	
	/**
	 * Reference to HTML DIV or other element where the map is placed 
	 * on the page. This property is set in the Map constructor.
	 * 
	 * @return String
	 */
	public native String getId() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.id;
	}-*/;
	
	/**
	 * Displays the InfoWindow on a map.
	 * 
	 * @return InfoWindow
	 */
	public native InfoWindow getInfoWindow() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.infoWindow;
	}-*/;
	
	/**
	 * When true, the key sequence of shift then click to recenter the map is enabled.
	 * 
	 * @return boolean
	 */
	public native boolean isClickRecenter() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.isClickRecenter;
	}-*/;
	
	/**
	 * When true, double click zoom is enabled. This allows a user to zoom and recenter
	 * the map by double clicking the mouse.
	 * 
	 * @return boolean
	 */
	public native boolean isDoubleClickZoom() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.isDoubleClickZoom;
	}-*/;
	
	/**
	 * When true, double click zoom is enabled. This allows a user to zoom and recenter
	 * the map by double clicking the mouse.
	 * 
	 * @return boolean
	 */
	public native boolean isKeyboardNavigation() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.isKeyboardNavigation;
	}-*/;
	
	/**
	 * When true, map panning is enabled using the mouse.
	 * 
	 * @return boolean
	 */
	public native boolean isPan() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.isPan;
	}-*/;
	
	/**
	 * When true, pan arrows are displayed around the edge of the map.
	 * 
	 * @return boolean
	 */
	public native boolean isPanArrows() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.isPanArrows;
	}-*/;
	
	/**
	 * When true, rubberband zoom is enabled. This allows users to draw
	 * a bounding box zoom area using the mouse.
	 * 
	 * @return boolean
	 */
	public native boolean isRubberBandZoom() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.isRubberBandZoom;
	}-*/;
	
	/**
	 * When true, the mouse scroll wheel zoom is enabled.
	 * 
	 * @return boolean
	 */
	public native boolean isScrollWheelZoom() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.isScrollWheelZoom;
	}-*/;
	
	/**
	 * When true, shift double click zoom is enabled. This allows a user to 
	 * zoom and recenter the map by shift + double clicking the mouse.
	 * 
	 * @return boolean
	 */
	public native boolean isShiftDoubleClickZoom() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.isShiftDoubleClickZoom;
	}-*/;
	
	/**
	 * When true, the zoom slider is displayed on the map.
	 * 
	 * @return boolean
	 */
	public native boolean isZoomSlider() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.isZoomSlider;
	}-*/;
	
	/**
	 * Array of current TiledMapServiceLayers and DynamicMapServiceLayers 
	 * added to the map.
	 * 
	 * @return JsArrayString
	 */
	public native JsArrayString getLayerIds() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.layerIds;
	}-*/;
	
	/**
	 * After the first layer is loaded, the value is set to true.
	 * 
	 * @return boolean
	 */
	public native boolean isLoaded() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.loaded;
	}-*/;	
	
	/**
	 * This point geometry in screen coordinates represent the top-left corner
	 * of the map container. This coordinate also acts as the origin for all
	 * screen coordinates returned by Map and GraphicsLayer events.
	 * 
	 * @return Point
	 */
	public native Point getPosition() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.position;
	}-*/;
	
	/**
	 * The spatial reference of the map.
	 * 
	 * @return SpatialReference
	 */
	public native SpatialReference getSpatialReference() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.spatialReference;
	}-*/;
	
	/**
	 * The current TimeExtent for the map. Use the setTimeExtent method to modify the time extent.
	 * 
	 * @return TimeExtent
	 */
	public native TimeExtent getTimeExtent() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.timeExtent;
	}-*/;
	
	/**
	 * Current width of the map in screen pixels.
	 * 
	 * @return int
	 */
	public native int getWidth() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.width;
	}-*/;
	
	
	
	/*
	 * Map Methods
	 */
	
	/**
	 * Adds an ESRI Layer to the map.
	 * 
	 * @param layer - Layer to be added to the map.
	 * @param index - A layer can be added at a specific index in the map.
	 */
	public native Layer addLayer(Layer layer, int index) /*-{
		return this.@com.esri.gwt.client.MapWidget::map.addLayer(layer, index);
	}-*/;
	
	/**
	 * Adds an ESRI Layer to the map.
	 * 
	 * @param layer - Layer to be added to the map.
	 */
	public native Layer addLayer(Layer layer) /*-{
		return this.@com.esri.gwt.client.MapWidget::map.addLayer(layer);
	}-*/;
	
	/**
	 * Adds multiple layers to a map. The array order corresponds to the layer order in the client side map. The onLayerAddResult
	 * event fires for each layer that is added. Once all the layers are added to the map the onLayersAddResult fires.
	 * 
	 * @param layers - Layers to be added to the map.
	 */
	public void addLayers(Layer[] layers) {
		JavaScriptObject arr = JavaScriptObject.createArray();
		for( int i = 0; i < layers.length; i++ ) Util.addObjectToArray(layers[i], arr);
		_addLayers(arr);
	};
	
	private native void _addLayers(JavaScriptObject layers) /*-{
		this.@com.esri.gwt.client.MapWidget::map.addLayers(layers);
	}-*/;
	
	/**
	 * Centers and zooms the map.
	 * 
	 * @param center - Centers the map on the specified x,y location.
	 * @param levelOrFactor - When using an ArcGISTiledMapServiceLayer, the map is zoomed 
	 * to the level specified. When using a DynamicMapServiceLayer, the map is zoomed in or 
	 * out by the specified factor. For example, use 0.5 to zoom in twice as far and 2.0 to zoom 
	 * out twice as far.
	 */
	public native void centerAndZoom(Point center, double levelOrFactor) /*-{
		this.@com.esri.gwt.client.MapWidget::map.centerAndZoom(center, levelOrFactor);
	}-*/;
	
	/**
	 * Recenters the map based on map coordinates as the center point.
	 * 
	 * @param center - Centers the map on the specified x,y location.
	 */
	public native void centerAt(Point center) /*-{
		this.@com.esri.gwt.client.MapWidget::map.centerAt(center);
	}-*/;
	
	/**
	 * Destroys the map instance.
	 */
	public void destroy() {
		_destroy();
		if( isAttached() ) removeFromParent();
	} 
	
	private native void _destroy() /*-{
		this.@com.esri.gwt.client.MapWidget::map.destroy();
	}-*/;
	
	/**
	 * Disallows clicking on a map to recenter it.
	 */
	public native void disableClickRecenter() /*-{
		this.@com.esri.gwt.client.MapWidget::map.disableClickRecenter();
	}-*/;
	
	/**
	 * Disallows double clicking on a map to zoom in a level and recenter the map.
	 */
	public native void disableDoubleClickZoom() /*-{
		this.@com.esri.gwt.client.MapWidget::map.disableDoubleClickZoom();
	}-*/;
	
	/**
	 * Disallows panning and zooming using the keyboard.
	 */
	public native void disableKeyboardNavigation() /*-{
		this.@com.esri.gwt.client.MapWidget::map.disableKeyboardNavigation();
	}-*/;
	
	/**
	 * Disallows all map navigation except the slider and pan arrows.
	 */
	public native void disableMapNavigation() /*-{
		this.@com.esri.gwt.client.MapWidget::map.disableMapNavigation();
	}-*/;
	
	/**
	 * Disallows panning a map using the mouse.
	 */
	public native void disablePan() /*-{
		this.@com.esri.gwt.client.MapWidget::map.disablePan();
	}-*/;
	
	/**
	 * Disallows zooming in or out on a map using a bounding box.
	 */
	public native void disableRubberBandZoom() /*-{
		this.@com.esri.gwt.client.MapWidget::map.disableRubberBandZoom();
	}-*/;
	
	/**
	 * Disallows zooming in or out on a map using the mouse scroll wheel.
	 */
	public native void disableScrollWheelZoom() /*-{
		this.@com.esri.gwt.client.MapWidget::map.disableScrollWheelZoom();
	}-*/;
	
	/**
	 * Disallows shift double clicking on a map to zoom in a level and recenter the map.
	 */
	public native void disableShiftDoubleClickZoom() /*-{
		this.@com.esri.gwt.client.MapWidget::map.disableShiftDoubleClickZoom();
	}-*/;
	
	/**
	 * Permits users to click on a map to recenter it.
	 */
	public native void enableClickRecenter() /*-{
		this.@com.esri.gwt.client.MapWidget::map.enableClickRecenter();
	}-*/;
	
	/**
	 * Permits users to double click on a map to zoom in a level and recenter the map
	 */
	public native void enableDoubleClickZoom() /*-{
		this.@com.esri.gwt.client.MapWidget::map.enableDoubleClickZoom();
	}-*/;
	
	/**
	 * Permits users to pan and zoom using the keyboard.
	 */
	public native void enableKeyboardNavigation() /*-{
		this.@com.esri.gwt.client.MapWidget::map.enableKeyboardNavigation();
	}-*/;
	
	/**
	 * Allows all map navigation.
	 */
	public native void enableMapNavigation() /*-{
		this.@com.esri.gwt.client.MapWidget::map.enableMapNavigation();
	}-*/;
	
	/**
	 * Permits users to pan a map using the mouse.
	 */
	public native void enablePan() /*-{
		this.@com.esri.gwt.client.MapWidget::map.enablePan();
	}-*/;
	
	/**
	 * Permits users to zoom in or out on a map using a bounding box.
	 */
	public native void enableRubberBandZoom() /*-{
		this.@com.esri.gwt.client.MapWidget::map.enableRubberBandZoom();
	}-*/;
	
	/**
	 * Permits users to zoom in or out on a map using the mouse scroll wheel. 
	 */
	public native void enableScrollWheelZoom() /*-{
		this.@com.esri.gwt.client.MapWidget::map.enableScrollWheelZoom();
	}-*/;
	
	/**
	 * Permits users to shift double click on a map to zoom in a level and recenter the map.
	 */
	public native void enableShiftDoubleClickZoom() /*-{
		this.@com.esri.gwt.client.MapWidget::map.enableShiftDoubleClickZoom();
	}-*/;
	
	/**
	 * Sets an InfoWindow's anchor when calling InfoWindow.show
	 * 
	 * @param screenCoords - The anchor point in screen units.
	 * @return String
	 */
	public native String getInfoWindowAnchor(Point screenCoords) /*-{
		return this.@com.esri.gwt.client.MapWidget::map.getInfoWindowAnchor(screenCoords);
	}-*/;
	
	/**
	 * Returns an individual layer of a map.
	 * 
	 * @param id - ID assigned to the layer.
	 * @return boolean
	 */
	public native Layer getLayer(String id) /*-{
		return this.@com.esri.gwt.client.MapWidget::map.getLayer(id);
	}-*/;

	/**
	 * Gets the current level of detail ID for the map. Valid only with 
	 * an ArcGISTiledMapService layer.
	 * 
	 * @return float
	 */
	public native float getLevel() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.getLevel();
	}-*/;

	/**
	 * Hides the pan arrows from the map.
	 */
	public native void hidePanArrows() /*-{
		this.@com.esri.gwt.client.MapWidget::map.hidePanArrows();
	}-*/;
	
	/**
	 * Hides the zoom slider from the map.
	 */
	public native void hideZoomSlider() /*-{
		this.@com.esri.gwt.client.MapWidget::map.hideZoomSlider();
	}-*/;
	
	/**
	 * Pans the map south.
	 */
	public native void panDown() /*-{
		this.@com.esri.gwt.client.MapWidget::map.panDown();
	}-*/;
	
	/**
	 * Pans the map west.
	 */
	public native void panLeft() /*-{
		this.@com.esri.gwt.client.MapWidget::map.panLeft();
	}-*/;
	
	/**
	 * Pans the map southwest.
	 */
	public native void panLowerLeft() /*-{
		this.@com.esri.gwt.client.MapWidget::map.panLowerLeft();
	}-*/;
	
	/**
	 * Pans the map southeast.
	 */
	public native void panLowerRight() /*-{
		this.@com.esri.gwt.client.MapWidget::map.panLowerRight();
	}-*/;
	
	/**
	 * Pans the map east.
	 */
	public native void panRight() /*-{
		this.@com.esri.gwt.client.MapWidget::map.panRight();
	}-*/;
	
	/**
	 * Pans the map north.
	 */
	public native void panUp() /*-{
		this.@com.esri.gwt.client.MapWidget::map.panUp();
	}-*/;
	
	/**
	 * Pans the map northwest.
	 */
	public native void panUpperLeft() /*-{
		this.@com.esri.gwt.client.MapWidget::map.panUpperLeft();
	}-*/;
	
	/**
	 * Pans the map northeast.
	 */
	public native void panUpperRight() /*-{
		this.@com.esri.gwt.client.MapWidget::map.panUpperRight();
	}-*/;

	/**
	 * Removes all layers from the map.
	 * 
	 * The map's extent, spatial reference, and tiling scheme if defined 
	 * do not change when the layers are removed. When the next layer is 
	 * added to the map, this layer is displayed at the same extent and 
	 * spatial reference.
	 */
	public native void removeAllLayers() /*-{
		return this.@com.esri.gwt.client.MapWidget::map.removeAllLayers();
	}-*/;

	/**
	 * Removes the specified layer from the map.
	 * 
	 * @param layer - Layer to be removed from the map.
	 */
	public native void removeLayer(Layer layer) /*-{
		this.@com.esri.gwt.client.MapWidget::map.removeLayer(layer);
	}-*/;

	/**
	 * Changes the layer order in the map.
	 * 
	 * @param layer - The layer to be moved.
	 * @param index - Refers to the location for placing the layer. The bottom 
	 * most layer has an index of 0.
	 */
	public native void reorderLayer(Layer layer, int index) /*-{
		this.@com.esri.gwt.client.MapWidget::map.reorderLayer(layer, index);
	}-*/;	
	
	/**
	 * Repositions the map DIV on the page. This method should be used after
	 * the map DIV has been repostioned.
	 */
	public native void reposition() /*-{
		this.@com.esri.gwt.client.MapWidget::map.reposition();
	}-*/;

	/**
	 * Resizes the map DIV. This method should be used after the map DIV has been resized.
	 */
	public native void resize() /*-{
		this.@com.esri.gwt.client.MapWidget::map.resize();
	}-*/;		
	
	/**
	 * Sets the extent of the map. The extent must be in the same spatial reference as the map.
	 * 
	 * @param extent - Sets the minx, miny, maxx, and maxy for a map.
	 * @param fit - When true, for maps that contain tiled map service layers, you are guaranteed 
	 * to have the input extent shown completely on the map.
	 */
	public native void setExtent(Extent extent, boolean fit) /*-{
		this.@com.esri.gwt.client.MapWidget::map.setExtent(extent, fit);
	}-*/;
	
	/**
	 * Sets the extent of the map. The extent must be in the same spatial reference as the map.
	 * 
	 * @param extent - Sets the minx, miny, maxx, and maxy for a map.
	 */
	public native void setExtent(Extent extent) /*-{
		this.@com.esri.gwt.client.MapWidget::map.setExtent(extent);
	}-*/;	
	
	/**
	 * Sets the map to the specified level ID. Zooms to the new level based on 
	 * the current map center point. Valid only with an ArcGISTiledMapService layer.
	 * 
	 * @param level - The level ID.
	 */
	public native void setLevel(int level) /*-{
		this.@com.esri.gwt.client.MapWidget::map.setLevel(level);
	}-*/;

	/**
	 * Sets the default cursor for the map. This cursor is shown whenever the mouse
	 * is pointed over the map, except when panning by dragging the map or using
	 * SHIFT+Drag to zoom. If not set, the map uses the platform-dependent default
	 * cursor, usually an arrow.
	 * 
	 * @param cursor - A standard CSS cursor value. Some common values include "default",
	 * "pointer", "crosshair", "text", "help", and "wait".
	 */
	public native void setMapCursor(String cursor) /*-{
		this.@com.esri.gwt.client.MapWidget::map.setMapCursor(cursor);
	}-*/;

	/**
	 * Sets the TimeExtent for the map. When the time extent is changed the onTimeExtentChange
	 * event fires. Time aware layers listen for this event and update to only display content for the
	 * current time extent.
	 * 
	 * @param timeExtent - Set the time extent for which data is displayed on the map.
	 */
	public native void setTimeExtent(TimeExtent timeExtent) /*-{
		this.@com.esri.gwt.client.MapWidget::map.setTimeExtent(timeExtent);
	}-*/;
	
	/**
	 * Set the time slider associated with the map.
	 * 
	 * @param timeSlider - The time slider dijit to associate with the map.
	 */
	public native void setTimeSlider(TimeSlider timeSlider) /*-{
		this.@com.esri.gwt.client.MapWidget::map.setTimeSlider(timeSlider);
	}-*/;
	
	/**
	 * Shows the pan arrows on the map.
	 */
	public native void showPanArrows() /*-{
		this.@com.esri.gwt.client.MapWidget::map.showPanArrows();
	}-*/;
	
	/**
	 * Shows the zoom slider on the map.
	 */
	public native void showZoomSlider() /*-{
		this.@com.esri.gwt.client.MapWidget::map.showZoomSlider();
	}-*/;
	
	/**
	 * Converts a single screen point or an array of screen points to map coordinates.
	 * 
	 * @param screenPoint - Converts screen coordinates to map coordinates.
	 * @return Point
	 */
	public final native Point toMap(Point screenPoint) /*-{
		return this.@com.esri.gwt.client.MapWidget::map.toMap(screenPoint);
	}-*/;

	/**
	 * Converts a single map point or an array of map points to screen coordinates.
	 * 
	 * @param mapPoint - Converts map coordinates to screen coordinates.
	 * @return Point
	 */
	public final native Point toScreen(Point mapPoint) /*-{
		return this.@com.esri.gwt.client.MapWidget::map.toScreen(mapPoint);
	}-*/;
	
	/*
	 * EVENTS
	 */
	
	/**
	 * Add handler for when map is clicked.
	 * 
	 * @param handler - Fires when a user single clicks on the map using the mouse and the mouse pointer\
	 * is within the map region of the HTML page.
	 */
	public native void addClickHandler(ClickHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onClick",
			function(event) {
				handler.@com.esri.gwt.client.event.ClickHandler::onClick(Lcom/esri/gwt/client/event/MouseEvent;)(event);
			}
		);
	}-*/;
	
	/**
	 * Add handler for when maps is double clicked
	 * 
	 * @param handler - Fires when a user double clicks on the map using the mouse and the mouse pointer is
	 * within the map region of the HTML page.
	 */
	public native void addDblClickHandler(DblClickHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onDblClick",
			function(event) {
				handler.@com.esri.gwt.client.event.DblClickHandler::onDblClick(Lcom/esri/gwt/client/event/MouseEvent;)(event);
			}
		);
	}-*/;
	
	/**
	 * Add Handler to listen to maps onResize Event
	 * 
	 * Fires when the extent of the map has changed.
	 * 
	 * @param handler
	 */
	public native void addExtentChangeHandler(MapExtentChangeHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onExtentChange",
			function(extent, delta, levelChange, lod) {
				handler.@com.esri.gwt.client.event.MapExtentChangeHandler::onMapExtentChange(Lcom/esri/gwt/client/geometry/Extent;Lcom/esri/gwt/client/geometry/Point;ZLcom/esri/gwt/client/layers/LOD;)(extent, delta, levelChange, lod);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires when a keyboard key is pressed.
	 */
	public native void addKeyDownHandler(MapKeyDownHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onKeyDown",
			function(event) {
				handler.@com.esri.gwt.client.event.MapKeyDownHandler::onKeyDown(Lcom/esri/gwt/client/event/KeyEvent;)(event);
			}
		);
	 }-*/;
	
	/**
	 * 
	 * @param handler - Fires when a keyboard key is released.
	 */
	public native void addKeyUpHandler(MapKeyUpHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onKeyUp",
			function(event) {
				handler.@com.esri.gwt.client.event.MapKeyUpHandler::onKeyUp(Lcom/esri/gwt/client/event/KeyEvent;)(event);
			}
		);
	 }-*/;
	
	/**
	 * 
	 * @param handler - Fires any time a layer is added to the map.
	 */
	public native void addLayerAddHandler(MapLayerAddHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onLayerAdd",
			function(layer) {
				handler.@com.esri.gwt.client.event.MapLayerAddHandler::onLayerAdd(Lcom/esri/gwt/client/layers/Layer;)(layer);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires after specified layer has been added to the map.
	 */
	public native void addLayerAddResultHandler(MapLayerAddResultHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onLayerAddResult",
			function(layer, error) {
				handler.@com.esri.gwt.client.event.MapLayerAddResultHandler::onLayerAddResult(Lcom/esri/gwt/client/layers/Layer;Lcom/esri/gwt/client/Error;)(layer, error);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires after the layer has been removed.
	 */
	public native void addLayerRemoveHandler(MapLayerRemoveHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onLayerRemove",
			function(layer) {
				handler.@com.esri.gwt.client.event.MapLayerRemoveHandler::onLayerRemove(Lcom/esri/gwt/client/layers/Layer;)(layer);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires when the map layer order has been changed.
	 */
	public native void addLayerReorderHandler(MapLayerReorderHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onLayerReordered",
			function(layer, index) {
				handler.@com.esri.gwt.client.event.MapLayerReorderHandler::onLayerReorder(Lcom/esri/gwt/client/layers/Layer;I)(layer, index);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires after all layers are added to the map.
	 */
	public native void addLayersAddResultHandler(MapLayersAddResultHandler handler)/*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onLayersAddResult",
			function(results) {
				handler.@com.esri.gwt.client.event.MapLayersAddResultHandler::onLayersAddResult(Lcom/google/gwt/core/client/JsArray;)(results);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires after all the layers have been removed.
	 */
	public native void addLayersRemovedHandler(MapLayersRemovedHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onLayersRemoved",
			function() {
				handler.@com.esri.gwt.client.event.MapLayersRemovedHandler::onLayersRemoved()();
			}
		);
	}-*/;
	
	/**
	 * Fires when all the layers have been reordered.
	 * 
	 * @param handler
	 */
	public native void addLayersReorderedHandler(MapLayersReorderedHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onLayersReordered",
			function(ids) {
				handler.@com.esri.gwt.client.event.MapLayersReorderedHandler::onLayersReordered(Lcom/google/gwt/core/client/JsArrayString;)(ids);
			}
		);
	}-*/;
	
	
	/**
	 * Add Handler to listen to maps onLoad Event
	 * 
	 * @param handler - Fires when the first or base layer has been successfully added to the map.
	 */
	public native void addLoadHandler(MapLoadHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onLoad",
			function(map) {
				handler.@com.esri.gwt.client.event.MapLoadHandler::onLoad()();
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires when a mouse button is pressed down and the mouse cursor is in the map region of the HTML page.
	 */
	public native void addMouseDownHandler(MouseDownHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onMouseDown",
			function(event) {
				handler.@com.esri.gwt.client.event.MouseDownHandler::onMouseDown(Lcom/esri/gwt/client/event/MouseEvent;)(event);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires while the mouse is being dragged until the mouse button is released.
	 */
	public native void addMouseDragHandler(MapMouseDragHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onMouseDrag",
			function(event) {
				handler.@com.esri.gwt.client.event.MapMouseDragHandler::onMouseDrag(Lcom/esri/gwt/client/event/MouseEvent;)(event);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires when a mouse button is released and the user stops dragging the mouse.
	 */
	public native void addMouseDragEndHandler(MapMouseDragEndHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onMouseDragEnd",
			function(event) {
				handler.@com.esri.gwt.client.event.MapMouseDragEndHandler::onMouseDragEnd(Lcom/esri/gwt/client/event/MouseEvent;)(event);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires when a mouse button is pressed down and the user starts to drag the mouse.
	 */
	public native void addMouseDragStartHandler(MapMouseDragStartHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onMouseDragStart",
			function(event) {
				handler.@com.esri.gwt.client.event.MapMouseDragStartHandler::onMouseDragStart(Lcom/esri/gwt/client/event/MouseEvent;)(event);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires any time the mouse pointer moves over the map region. A common use for this event is to show the current x,y
	 * coordinate of the map as the user moves the mouse pointer.
	 */
	public native void addMouseMoveHandler(MouseMoveHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onMouseMove",
			function(event) {
				handler.@com.esri.gwt.client.event.MouseMoveHandler::onMouseMove(Lcom/esri/gwt/client/event/MouseEvent;)(event);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires when the mouse moves out of the map region of the HTML page.
	 */
	public native void addMouseOutHandler(MouseOutHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onMouseOut",
			function(event) {
				handler.@com.esri.gwt.client.event.MouseOutHandler::onMouseOut(Lcom/esri/gwt/client/event/MouseEvent;)(event);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires when the mouse moves into the map region of the HTML page.
	 */
	public native void addMouseOverHandler(MouseOverHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onMouseOver",
			function(event) {
				handler.@com.esri.gwt.client.event.MouseOverHandler::onMouseOver(Lcom/esri/gwt/client/event/MouseEvent;)(event);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires when the mouse button is released and the mouse pointer is within the map region of the HTML page.
	 */
	public native void addMouseUpHandler(MouseUpHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onMouseUp",
			function(event) {
				handler.@com.esri.gwt.client.event.MouseUpHandler::onMouseUp(Lcom/esri/gwt/client/event/MouseEvent;)(event);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires when the mouse wheel is scrolled.
	 */
	public native void addMouseWheelHandler(MouseWheelHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onMouseWheel",
			function(event) {
				handler.@com.esri.gwt.client.event.MouseWheelHandler::onMouseWheel(Lcom/esri/gwt/client/event/MouseEvent;)(event);
			}
		);
	}-*/;
	
	/**
	 * Add Handler to listen to maps onPan Event
	 *  
	 * @param handler -  Fires during the pan process.
	 */
	public native void addPanHandler(MapPanHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onPan",
			function(extent, delta) {
				handler.@com.esri.gwt.client.event.MapPanHandler::onMapPan(Lcom/esri/gwt/client/geometry/Extent;Lcom/esri/gwt/client/geometry/Point;)(extent, delta);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires when the pan is complete.
	 */
	public native void addPanEndHandler(MapPanEndHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onPanEnd",
			function(extent, endPoint) {
				handler.@com.esri.gwt.client.event.MapPanEndHandler::onMapPanEnd(Lcom/esri/gwt/client/geometry/Extent;Lcom/esri/gwt/client/geometry/Point;)(extent, endPoint);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires when a user commences panning.
	 */
	public native void addPanStartHandler(MapPanStartHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onPanStart",
			function(extent, startPoint) {
				handler.@com.esri.gwt.client.event.MapPanStartHandler::onMapPanStart(Lcom/esri/gwt/client/geometry/Extent;Lcom/esri/gwt/client/geometry/Point;)(extent, startPoint);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires when the map DIV is repositioned.
	 */
	public native void addRepositionHandler(MapRepositionHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onReposition",
			function(x, y) {
				handler.@com.esri.gwt.client.event.MapRepositionHandler::onMapReposition(FF)(x, y);
			}
		);
	}-*/;
	
	/**
	 * Add Handler to listen to maps onResize Event
	 * 
	 * @param handler - Fires when the Div containing the map has been resized.
	 */
	public native void addResizeHandler(MapResizeHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onResize",
			function(extent, height, width) {
				handler.@com.esri.gwt.client.event.MapResizeHandler::onMapResize(Lcom/esri/gwt/client/geometry/Extent;II)(extent, height, width);
			}
		);
	}-*/;

	/**
	 * 
	 * @param handler - Fires when the map's timeExtent property is set.
	 */
	public native void addTimeExtentChange(TimeExtentChangeHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onTimeExtentChange",
			function(timeExtent) {
				handler.@com.esri.gwt.client.event.TimeExtentChangeHandler::onTimeExtentChange(Lcom/esri/gwt/client/TimeExtent;)(timeExtent);
			}
		);
	}-*/;
	
	/**
	 * Add Handler to listen to maps onUnload Event
	 * 
	 * @param handler - Fires when the page is refreshed.
	 */
	public native void addUnloadHandler(MapUnloadHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onUnload",
			function(map) {
				handler.@com.esri.gwt.client.event.MapUnloadHandler::onUnload(Lcom/google/gwt/core/client/JavaScriptObject;)(map);
			}
		);
	}-*/;

	/**
	 * 
	 * @param handler - Fires during the zoom process.
	 */
	public native void addZoomHandler(MapZoomHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onZoom",
			function(extent, zoomFactor, anchor) {
				handler.@com.esri.gwt.client.event.MapZoomHandler::onZoom(Lcom/esri/gwt/client/geometry/Extent;FLcom/esri/gwt/client/geometry/Point;)(extent, zoomFactor, anchor);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires when the zoom is complete.
	 */
	public native void addZoomEndHandler(MapZoomEndHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onZoomEnd",
			function(extent, zoomFactor, anchor, level)  {
				handler.@com.esri.gwt.client.event.MapZoomEndHandler::onZoomEnd(Lcom/esri/gwt/client/geometry/Extent;FLcom/esri/gwt/client/geometry/Point;I)(extent, zoomFactor, anchor, level);
			}
		);
	}-*/;
	
	/**
	 * 
	 * @param handler - Fires when a user commences zooming.
	 */
	public native void addZoomStartHandler(MapZoomStartHandler handler) /*-{
		$wnd.dojo.connect(
			this.@com.esri.gwt.client.MapWidget::map,
			"onZoomStart",
			function(extent, zoomFactor, anchor, level) {
				handler.@com.esri.gwt.client.event.MapZoomStartHandler::onZoomStart(Lcom/esri/gwt/client/geometry/Extent;FLcom/esri/gwt/client/geometry/Point;I)(extent, zoomFactor, anchor, level);
			}
		);
	}-*/;
	
	public static class Options extends JavaScriptObject {
		
		protected Options() {}
		
		public static Options create() {
			return JavaScriptObject.createObject().cast();
		};
		
		/**
		 * When true, graphics are displayed during panning. When false, the graphics 
		 * are turned off during pan movement. Setting to false may improve performance 
		 * in Internet Explorer. The default is true. 
		 * 
		 * @param displayGraphics
		 */
		public final native void setDisplayGraphicsOnPan(boolean displayGraphics) /*-{
			this["displayGraphicsOnPan"] = displayGraphics;
		}-*/;
		
		/**
		 * If provided, the extent and projection of the map is set to the properties of 
		 * Extent. Once the projection is set, all layers must be able to be drawn in the 
		 * defined projection. The tiling scheme of an ArcGISTiledMapServiceLayer must 
		 * match the projection of the map. The projection of the extent is honored as 
		 * of v1.2 when used in the map constructor.
		 * 
		 * @param extent
		 */
		public final native void setExtent(Extent extent) /*-{
			this["extent"] = extent;
		}-*/;
		
		/**
		 * When true, for maps that contain tiled map service layers, you are guaranteed
		 * to have the initial extent defined using the extent constructor option shown
		 * completely on the map. As of v2.0 
		 * 
		 * @param fitExtent
		 */
		public final native void fitExtent(boolean fitExtent) /*-{
			this["fitExtent"] = fitExtent;
		}-*/;
		
		/**
		 * By default the map creates and uses an out-of-the-box InfoWindow. If you create
		 * a custom implementation using InfoWindowBase specify it using this option.
		 * 
		 * @param infoWindow
		 */
		public final void setInfoWindow(InfoWindowBase infoWindow) {
			_setInfoWindow(infoWindow.getInfoWindowBaseObject());
		}
		
		private final native void _setInfoWindow(JavaScriptObject infoWindow) /*-{
			this["infoWindow"] = infoWindow;
		}-*/;
		
		// TODO: setLayer... this looks nice, how can we use it? docs suck.
		
		/**
		 * If provided, the map is initialized with the specified levels of detail. 
		 * All ArcGISTiledMapServiceLayers added to the map must have matching 
		 * resolutions in order to be displayed. (As of v1.2) 
		 * 
		 * @param lods
		 */
		public final void setLods(List<LOD> lods) {
			JavaScriptObject array = JavaScriptObject.createArray();
			for( LOD lod: lods ) Util.addObjectToArray(lod, array);
			setLods(array);
		}
		
		private final native void setLods(JavaScriptObject lods) /*-{
			this["lods"] = lods;
		}-*/;
		
		/**
		 * Display the esri logo on the map. The default value is true.
		 * 
		 * @param logo
		 */
		public final native void setLogo(boolean logo) /*-{
			this["logo"] = logo;
		}-*/;

		/**
		 * Displays pan buttons on map. When false, the pan arrows never
		 * display. The default is false. 
		 * 
		 * @param nav
		 */
		public final native void setNav(boolean nav) /*-{
			this["nav"] = nav;
		}-*/;
		
		/**
		 * The default behavior is to show an InfoWindow if the Graphic 
		 * has a defined InfoTemplate when the user clicks on the graphic. 
		 * When true, this default behavior is honored. The default value 
		 * is true. 
		 * 
		 * @param show
		 */
		public final native void showInfoWindowOnClick(boolean show) /*-{
			this["showInfoWindowOnClick"] = show;
		}-*/;
		
		/**
		 * Displays a slider on the map. When false, the slider never displays. 
		 * Default is true.
		 * 
		 * @param show
		 */
		public final native void showSlider(boolean show) /*-{
			this["slider"] = show;
		}-*/;

		/**
		 * When true, supports continuous pan across the dateline.
		 * Wraparound is supported in the following cases:
		 * 
		 * - Map spatial reference is WGS84 or Web Mercator
		 * - The tiling scheme is either the pre-9.3 ArcGIS Online (4326)
		 * tiling scheme or ArcGIS/Google Maps/ Bing tiling scheme.
		 * - Dynamic services must be version 10 or greater.
		 * 
		 * The default value is false.
		 * 
		 * @param wrap
		 */
		public final native void wrapAround180(boolean wrap) /*-{
			this["wrapAround180"] = wrap;
		}-*/;
	}

} 
