package com.esri.gwt.client.layers;

import com.esri.gwt.client.event.MapImageExportHandler;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.core.client.JsArrayInteger;
import com.google.gwt.core.client.JsArrayString;
import com.google.gwt.core.client.JsArray;

/**
 * Allows you to work with a dynamic map service resource exposed by the 
 * ArcGIS Server REST API. A dynamic map service generates images on the
 * fly. For cached map services, @see com.esri.gwt.client.layers.ArcGISTiledMapServiceLayer.
 *
 * If the first layer added to the map is an ArcGISDynamicMapServiceLayer,
 * the map will take on the projection of this layer. 
 * 
 * @author Justin Merz
 */
public class ArcGISDynamicMapServiceLayer extends DynamicMapServiceLayer {
	
	protected ArcGISDynamicMapServiceLayer() {}
	
	/**
	 * Creates a new ArcGISDynamicMapServiceLayer object. A URL is a required
	 * parameter. 
	 * 
	 * @param uri - URL to the ArcGIS Server REST resource that represents a map service.
	 * @return ArcGISDynamicMapServiceLayer
	 */
	public native final static ArcGISDynamicMapServiceLayer create(String uri) /*-{
		return new $wnd.esri.layers.ArcGISDynamicMapServiceLayer(uri);
	}-*/;
	
	/**
	 * Creates a new ArcGISDynamicMapServiceLayer object. A URL is a required
	 * parameter. This layer also takes some optional parameters.
	 * 
	 * @param uri - URL to the ArcGIS Server REST resource that represents a map service.
	 * @param options - Optional parameters.
	 * @return ArcGISDynamicMapServiceLayer
	 */
	public native final static ArcGISDynamicMapServiceLayer create(String uri, Options options) /*-{
		return new $wnd.esri.layers.ArcGISDynamicMapServiceLayer(uri, options);
	}-*/;
	
	/**
	 * Capabilities of the map service, possible values are Map, Query and Data. Only available if the map service
	 * is published using ArcGIS Server version 10 or greater.
	 * 
	 * @return String
	 */
	public native final String getCapabilities() /*-{
		return this.capabilities;
	}-*/;
	
	/**
	 * Copyright string as defined by the map service.
	 * 
	 * @return String
	 */
	public native final String getCopyright() /*-{
		return this.copyright;
	}-*/;
	
	/**
	 * Map description as defined by the map service.
	 * 
	 * @return String
	 */
	public native final String getDescription() /*-{
		return this.description;
	}-*/;
	
	/**
	 * When true, images are always requested from the server and the browser's cache is ignored. This should
	 *  be used when the data supporting the map service changes frequently.
	 * 
	 * @return boolean
	 */
	public native final boolean isDisabledClientCaching() /*-{
		return this.disableClientCaching;
	}-*/;
	
	/**
	 * The output dpi of the dynamic map service layer.
	 * 
	 * @return int
	 */
	public native final int getDpi() /*-{
		return this.dpi;
	}-*/;
	
	/**
	 * The output image type. As of ArcGIS Server 9.3.1, the list of supported image formats is included in the 
	 * description of Map Services in Services Directory under "Supported Image Format Types". In addition, 
	 * as of ArcGIS Server 9.3.1, optimized map services can produce true PNG32 images.
	 * 
	 * Known values: png | png8 | png24 | png32 | jpg | pdf | bmp | gif | svg 
	 * 
	 * @return String
	 */
	public native final String getImageFormat() /*-{
		return this.imageFormat;
	}-*/;
	
	/**
	 * Whether or not background of dynamic image is transparent.
	 * 
	 * @return boolean
	 */
	public native final boolean imageTransparency() /*-{
		return this.imageTransparency;
	}-*/;
	
	/**
	 * Sets the layer definitions used to filter the features of individual layers in the map service. Layer 
	 * definitions with semicolons or colons are supported at version 2.0 if using a map service published 
	 * using ArcGIS Server 10.0
	 * 
	 * @return JsArrayString
	 */
	public native final JsArrayString getLayerDefinitions() /*-{
		return this.layerDefinitions;
	}-*/;
	
	/**
	 * Returns the available layers in service and their default visibility.
	 * 
	 * @return JsArray<LayerInfo>
	 */
	public native final JsArray<LayerInfo> getLayerInfos() /*-{
		return this.layerInfos;
	}-*/;
	
	/**
	 * Returns the current layer time options if applicable. Use the setLayerTimeOptions method
	 * to modify the time options. (As of v2.0)
	 * 
	 * @return JsArray<LayerTimeOptions>
	 */
	public native final JsArray<LayerTimeOptions> getTimeLayerOptions() /*-{
		return this.TimeLayerOptions
	}-*/;
	
	/**
	 * Default units of the layer as defined by the service. If the layer is the base map, the map is in these units.
	 * 
	 * @return String
	 */
	public native final String getUnits() /*-{
		return this.units;
	}-*/;
	
	/**
	 * When true, the image is saved to the server, and a JSON formatted response is sent to the client with the URL 
	 * location of the image. A second call is automatically made to the server to retrieve the image. This value is set 
	 * in the constructor using useMapImage. Deprecated at v2.0. Prior to v2.0 this option was used when some 
	 * export map requests exceeded the limit of 2000 characters and it configured the map to use the proxy all the 
	 * time. This is not the optimal behavior so at v2.0 the switch to use the proxy will automatically happen for long 
	 * requests.
	 * 
	 * @deprecated
	 * @return boolean
	 */
	public native final boolean useMapImage() /*-{
		return this.useMapImage;
	}-*/;
	
	/**
	 * The version of ArcGIS Server where the map service is published.
	 * 
	 * @return float
	 */
	public native final float getVersion() /*-{
		return this.version;
	}-*/;	
	
	/**
	 * Gets the visible layers of the exported map.
	 * 
	 * @return JsArrayInteger
	 */
	public native final JsArrayInteger getVisibleLayers() /*-{
		return this.visibleLayers;
	}-*/;
	
	/**
	 * Exports a map using values as specified by ImageParameters.
	 * 
	 * @param params - Input parameters assigned before exporting the map image.
	 */
	public native final void exportMapImage(ImageParameters params) /*-{
			this.exportMapImage(params);
	}-*/;
	
	/**
	 * Exports a map using values as specified by ImageParameters. On completion, MapImage is returned.
	 * 
	 * @param params - Input parameters assigned before exporting the map image.
	 * @param handler - The function to call when the method has completed. The arguments in the function
	 * are the same as the onMapImageExport event.
	 */
	public native final void exportMapImage(ImageParameters params, MapImageExportHandler handler) /*-{
		var gwtCallback = null;
		if( handler != null ){
			gwtCallback = function(mapImage) {
				handler.@com.esri.gwt.client.event.MapImageExportHandler::onMapImageExport(Lcom/esri/gwt/client/layers/MapImage;)(mapImage);
			}
		}
		this.exportMapImage(params, gwtCallback);
	}-*/;
	
	/**
	 * Resets all layer definitions to those defined in the service.
	 */
	public native final void setDefaultLayerDefinitions() /*-{
		this.setDefaultLayerDefinitions();
	}-*/;
	
	/**
	 * Clears the visible layers as defined in setVisibleLayers, and resets to the default layers of the map service.
	 */
	public native final void setDefaultVisibleLayers() /*-{
		this.setDefaultVisibleLayers();
	}-*/;
	
	
	/**
	 * Sets whether images are always requested from the server and the browser's cache is ignored. This should
	 * be used when the data supporting the map service changes frequently.
	 * 
	 * @param disable - When true, client side caching is disabled.
	 */
	public native final void setDisableClientCaching(boolean disable) /*-{
		this.setDisableClientCaching(disable);
	}-*/;
	
	/**
	 * Sets the dpi of the exported map. The default value is 96.
	 * 
	 * @param dpi - DPI value.
	 */
	public native final void setDPI(int dpi) /*-{
		this.setDPI(dpi);
	}-*/;
	
	/**
	 * Sets the image format of the exported map.
	 * 
	 * @param imageFormat - Valid values are png | png8 | png24 | jpg | pdf | bmp | gif | svg.
	 */
	public native final void setImageFormat(String imageFormat) /*-{
		this.setImageFormat(imageFormat);
	}-*/;
	
	/**
	 * Sets the background of a dynamic image to transparent.
	 * 
	 * @param transparent - The default is "true".
	 */
	public native final void setImageTransparency(boolean transparent) /*-{
		this.setImageTransparency(transparent);
	}-*/;
	
	/**
	 * Sets the layer definitions used to filter the features of individual layers in the map service. If the layer 
	 * definition expression is longer than approximately 2000 characters you will need to set up a proxy page. 
	 * Prior to v2.0, you also need to set useMapImage to true.
	 * 
	 * Layer definitions with semicolons or colons are supported if using a map service published using ArcGIS Server 10.
	 * 
	 * @param layerDefinitions - An array containing each layer's definition.
	 */
	public native final void setLayerDefinitions(LayerDefinitions layerDefinitions) /*-{
		this.setLayerDefinitions(layerDefinitions);
	}-*/;
	
	/**
	 * Sets the visible layers of the exported map. By default, the visible layers are as defined by the default visibility in LayerInfo.
	 * 
	 * @param ids - Array of layer IDs.
	 */
	public native final void setVisibleLayers(JsArrayInteger ids) /*-{
		this.setVisibleLayers(ids);
	}-*/;
	
	/**
	 * Fires when the map export is completed.
	 * 
	 * @param handler - Contains the properties of a dynamically generated map image. 
	 */
	public native final void addMapImageExportHandler(MapImageExportHandler handler) /*-{
		var ref = this;
		if( !this._gwt_MapImageExportHandlers ) {
			this["_gwt_MapImageExportHandlers"] = [];
			this.onMapImageExport = function(mapImage) {
				for( var i in ref._gwt_MapImageExportHandlers ) {
					ref._gwt_MapImageExportHandlers[i].@com.esri.gwt.client.event.MapImageExportHandler::onMapImageExport(Lcom/esri/gwt/client/layers/MapImage;)(mapImage);
				}
			}
		}
		this._gwt_MapImageExportHandlers.push(handler);	
	}-*/;
	
	/**
	 * Layer Options
	 */
	public static class Options extends JavaScriptObject {
	
		protected Options() {}
		
		/**
		 * Create a new options object for ArcGISDynamicMapServiceLayer
		 * 
		 * @return Options
		 */
		public static final Options create() {
			return JavaScriptObject.createObject().cast();
		};
		
		/**
		 * Set the layer id.
		 * 
		 * @param id - Id to assign to the layer. If not assigned, esri.Map assigns value. 
		 */
		public final native void setId(String id) /*-{
			this["id"] = id;
		}-*/;
		
		/**
		 * Set the layers image parameters
		 * 
		 * @param imageParameters - Represents the image parameter options. 
		 */
		public final native void setImageParameters(ImageParameters imageParameters) /*-{
			this["imageParameters"] = imageParameters;
		}-*/;
		
		/**
		 * Set the initial opacity
		 * 
		 * @param opacity - Initial opacity or transparency of layer. Values range from 0.0 to 1.0,
		 *  where 0.0 is 100% transparent and 1.0 has no transparency. The default value is 1.0. 
		 */
		public final native void setOpacity(double opacity) /*-{
			this["opacity"] = opacity;
		}-*/;
		
		/**
		 * Set if should use map image.
		 * 
		 * @deprecated
		 * @param use - By default, images are exported in MIME format, and the image is streamed
		 *  to the client. When useMapImage is set to true, the image is saved to the server, and a JSON
		 *  formatted response is sent to the client with the URL location of the image. A second call is 
		 *  automatically made to the server to retrieve the image. This methodology is required for secure 
		 *  services that go through a proxy server. In this situation, a map image cannot be streamed. 
		 *  Deprecated at v2.0. Prior to v2.0 this option was used to configure the map to use the proxy all 
		 *  the time. This is not the optimal behavior so at v2.0 the switch to use the proxy will automatically 
		 *  happen if needed.
		 */
		public final native void useMapImage(boolean use) /*-{
			this["useMapImage"] = use;
		}-*/;
		
		/**
		 * Set if layer should be initially visible.
		 * 
		 * @param isVisible - Initial visibility of the layer. Default is true. 
		 */
		public final native void setVisible(boolean isVisible) /*-{
			this["visible"] = isVisible;
		}-*/; 
	
	}
	
	// TODO: don't think you can assign arrays like this, but just following the docs
	/**
	 * Example:
	 *  <code>
	 *  layerDefinitions.setLayerDefinition(0, "POPULATION > 5000000");
     *  layerDefinitions.setLayerDefinition(0, "AREA > 100000");
     *  dynamicMapServiceLayer.setLayerDefinitions(layerDefinitions);
     *  </code>
     * 
     * Note that the array indices 0 and 5 are valid layer IDs, and POPULATION and AREA are valid attribute fields 
     * associated with the respective layers. Definition expressions for layers that are currently not visible will be 
     * ignored by the server.
	 */
	public static class LayerDefinitions extends JavaScriptObject {
		
		protected LayerDefinitions() {}
		
		public static LayerDefinitions create() {
			return JavaScriptObject.createArray().cast();
		}
		
		/**
		 * Add layer definition for specified index
		 * 
		 * @param index - index of layer
		 * @param definition - String containing the definition
		 */
		public final native void setLayerDefinition(int index, String definition) /*-{
			this[index] =definition
		}-*/;
	};
	
}
