package com.esri.gwt.client.layers;

import java.util.List;

import com.esri.gwt.client.Util;
import com.esri.gwt.client.event.MapImageExportHandler;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.core.client.JsArray;
import com.google.gwt.core.client.JsArrayNumber;

/**
 * Allows you to work with an image map service resource exposed by the ArcGIS Server REST API.
 * 
 * @author Justin Merz
 */
public class ArcGISImageServiceLayer extends DynamicMapServiceLayer {
	
	protected ArcGISImageServiceLayer() {}
	
	/**
	 * Creates a new ArcGISImageServiceLayer object. A URL is a required parameter. This layer also takes some optional parameters.
	 * 
	 * @param uri - URL to the ArcGIS Server REST resource that represents a map service.
	 * @param options - Optional parameters.
	 * @return ArcGISImageServiceLayer
	 */
	public native static ArcGISImageServiceLayer create(String uri, Options options) /*-{
		return new $wnd.esri.layers.ArcGISImageServiceLayer(uri, options);
	}-*/;

	/**
	 * Creates a new ArcGISImageServiceLayer object. A URL is a required parameter.
	 * 
	 * @param uri - URL to the ArcGIS Server REST resource that represents a map service.
	 * @return ArcGISImageServiceLayer
	 */
	public native static ArcGISImageServiceLayer create(String uri) /*-{
		return new $wnd.esri.layers.ArcGISImageServiceLayer(uri);
	}-*/;

	/**
	 * Number of bands in ArcGISImageServiceLayer.
	 * 
	 * @return int
	 */
	public native final double getBandCount() /*-{
		return this.bandCount;
	}-*/;
	
	/**
	 * Array of current band selections.
	 * 
	 * @return JsArrayNumber
	 */
	public native final JsArrayNumber getBandIds() /*-{
		return this.bandIds;
	}-*/;
	
	/**
	 * The raster bands that the raster dataset is composed of and their 
	 * statistics. The length of this array gives the number of bands and 
	 * the array index represents the band ID.
	 * 
	 * @return JsArray<Band>
	 */
	public native final JsArray<Band> getBands() /*-{
		return this.bands;
	}-*/;
	
	/**
	 * Current compression quality value. The compression quality controls how much 
	 * loss the image will be subjected to. Only valid with JPG image types.
	 * 
	 * @return float
	 */
	public native final float getCompressionQuality() /*-{
		return this.compressionQuality;
	}-*/;
	
	/**
	 * Copyright string as defined by the image service.
	 * 
	 * @return String
	 */
	public native final String getCopyrightText() /*-{
		return this.copyrightText;
	}-*/;
	
	/**
	 * Copyright string as defined by the image service.
	 * 
	 * @return String
	 */
	public native final String getDescription() /*-{
		return this.description;
	}-*/;
	
	/**
	 * When true, images are always requested from the server and the browser's cache is 
	 * ignored. This should be used when the data supporting the map service changes frequently.
	 * 
	 * @return boolean
	 */
	public native final boolean isDisabledClientCaching() /*-{
		return this.disableClientCaching;
	}-*/;

	/**
	 * The output image type.
	 * 
	 * Known values: png8 | png24 | jpg | bmp | gif
	 * 
	 * Default value: png8 
	 * 
	 * @return String
	 */
	public native final String getFormat() /*-{
		return this.format;
	}-*/;

	/**
	 * Current interpolation method. The interpolation method affects how the raster dataset is 
	 * transformed when it undergoes warping or when it changes coordinate space.
	 * 
	 * @return String
	 */
	public native final String getInterpolation() /*-{
		return this.interpolation;
	}-*/;
	
	/**
	 * Specifies the mosaic rule when defining how individual images should be mosaicked.
	 * 
	 * @return MosaicRule
	 */
	public native final MosaicRule getMosaicRule () /*-{
		return this.mosaicRule;
	}-*/;
	
	/**
	 * Size of pixel in X direction.
	 * 
	 * @return float
	 */
	public native final float getPixelSizeX() /*-{
		return this.pixelSizeX;
	}-*/;
	
	/**
	 * Size of pixel in Y direction.
	 * 
	 * @return float
	 */
	public native final float getPixelSizeY() /*-{
		return this.pixelSizeY;
	}-*/;
	
	// TODO: test this.  Docs says int's, but example return values seem to be strings
	/**
	 * The pixel type of the image service. Pertains to the type of values stored in the raster, such as signed integer, 
	 * unsigned integer, or floating point. Integers are whole numbers, whereas floating points have decimals.
	 * 
	 * Known values: C64 | C128 | F32 | F64 | S8 | S16 | S32 | U1 | U2 | U4 | U8 | U16 | U32 | UNKNOWN 
	 * 
	 * @return String
	 */
	public native final String getPixelType() /*-{
		return this.pixelType;
	}-*/;
	
	/**
	 *  Exports a map using values as specified by ImageServiceParameters. On completion, MapImage is returned.
	 */
	public native final void exportMapImage() /*-{
			this.exportMapImage();
	}-*/;
	
	/**
	 * Exports a map using values as specified by ImageServiceParameters. On completion, MapImage is returned.
	 * 
	 * @param params - Input parameters assigned before exporting the map image.
	 * @param handler - The function to call when the method has completed. The arguments in the function are 
	 * the same as the onMapImageExport event.
	 */
	public native final void exportMapImage(ImageServiceParameters params, MapImageExportHandler handler) /*-{
		var gwtCallback = null;
		if( handler != null ){
			gwtCallback = function(mapImage) {
				handler.@com.esri.gwt.client.event.MapImageExportHandler::onMapImageExport(Lcom/esri/gwt/client/layers/MapImage;)(mapImage);
			}
		}
		this.exportMapImage(params, gwtCallback);
	}-*/;
	
	/**
	 * Sets the R,G,B of the exported image to the appropriate ImageService Band ID. BandIds are zero based.
	 * 
	 * @param bandIds - Array of band IDs to use in the exported image.
	 */
	public final void setBandIs(List<Integer> bandIds) {
		JavaScriptObject arr = JavaScriptObject.createArray();
		for( int id: bandIds ) Util.addIntToArray(id, arr);
		setBandIds(arr);
	}
	
	private native final void setBandIds(JavaScriptObject arr) /*-{
		this.bandIds = arr;
	}-*/;
	
	/**
	 * Sets the compression quality of the exported image. Only valid with JPG image format.
	 * 
	 * @param quality - Sets the compression quality of the exported image. Only valid with JPG image format.
	 */
	public native final void setCompressionQuality(float quality) /*-{
		this.setCompressionQuality(quality);
	}-*/;
	
	/**
	 * Sets whether images are always requested from the server and the browser's cache is ignored. This should
	 *  be used when the data supporting the map service changes frequently.
	 * 
	 * @param disable - When true, browser client side caching is disabled.
	 */
	public native final void setDisableClientCaching(boolean disable) /*-{
		this.setDisableClientCaching(disable);
	}-*/;
	
	/**
	 * Set the image format.
	 * 
	 * @param imageFormat - Valid values are png | png8 | png24 | jpg | pdf | bmp | gif | svg.
	 */
	public native final void setImageFormat(String imageFormat) /*-{
		this.setImageFormat(imageFormat);
	}-*/;
	
	/**
	 * Sets the interpolation method. The interpolation method affects how the raster dataset is transformed
	 *  when it undergoes warping or when it changes coordinate space.
	 *  
	 *  @param interpolation - Interpolation value defined in ImageServiceParameters Constants Table.
	 */
	public native final void setInterpolation(String interpolation) /*-{
		this.setInterpolation(interpolation);
	}-*/;
	
	/**
	 * Sets the mosaic rule of the layer to the specified value. The mosaic rule defines how individual images 
	 * should be mosaicked. Refreshes the layer to reflect the mosaic rule.
	 * 
	 * @param mosaicRule - The mosaic rule.
	 */
	public native final void setMosaicRule(MosaicRule mosaicRule) /*-{
		this.setMosaicRule(mosaicRule);
	}-*/;
	
	/**
	 * Sets the rendering rule of the layer to the given value. Refreshes the laeyr to reflect the rendering rule.
	 * 
	 * @param renderingRule - The new rendering rule.
	 */
	public native final void setRenderingRule(RasterFunction renderingRule) /*-{
		this.setRenderingRule(renderingRule);
	}-*/;
	
	/**
	 * Fires when the map export is completed.
	 * 
	 * @param handler - Contains the properties of a dynamically generated map image. 
	 */
	public native final void addMapImageExportHandler(MapImageExportHandler handler) /*-{
		var ref = this;
		if( !this._gwt_MapImageExportHandlers ) {
			this["_gwt_MapImageExportHandlers"] = [];
			this.onMapImageExport = function(mapImage) {
				for( var i in ref._gwt_MapImageExportHandlers ) {
					ref._gwt_MapImageExportHandlers[i].@com.esri.gwt.client.event.MapImageExportHandler::onMapImageExport(Lcom/esri/gwt/client/layers/MapImage;)(mapImage);
				}
			}
		}
		this._gwt_MapImageExportHandlers.push(handler);	
	}-*/;
	
	/**
	 * Constructor options for the ArcGISImageServiceLayer
	 */
	public static class Options extends JavaScriptObject {
		
		protected Options() {}
		
		/**
		 * Create new Options class for ArcGISImageServiceLayer
		 * 
		 * @return Options
		 */
		public static Options create() {
			return JavaScriptObject.createObject().cast();
		};
		
		/**
		 * Set id for layer
		 * 
		 * @param id - Id to assign to the layer. If not assigned, esri.Map assigns value. 
		 */
		public final native void setId(String id) /*-{
			this["id"] = id;
		}-*/;
		
		/**
		 * Set ImageServiceParameters for layer 
		 * 
		 * @param imageServiceParameters - The image service parameter options used when exporting an Image Service layer. 
		 */
		public final native void setImageServiceParameters(ImageServiceParameters imageServiceParameters) /*-{
			this["imageServiceParameters"] = imageServiceParameters;
		}-*/;
		
		/**
		 * Set opacity for layer
		 * 
		 * @param opacity - Initial opacity or transparency of layer. Values range from 0.0 to 1.0, where 0.0 is 100% transparent and
		 * 1.0 has no transparency. The default value is 1.0. 
		 */
		public final native void setOpacity(float opacity) /*-{
			this["opacity"] = opacity;
		}-*/;
		
		/**
		 * Set visible
		 * 
		 * @param isVisible - Initial visibility of the layer. Default is true.
		 */
		public final native void setVisible(boolean isVisible) /*-{
			this["visible"] = isVisible;
		}-*/; 
	
	}
	
}
