package com.esri.gwt.client.tasks;

import com.esri.gwt.client.callback.AddressToLocationsCallback;
import com.esri.gwt.client.callback.LocationToAddressCallback;
import com.esri.gwt.client.dojo.Deferred;
import com.esri.gwt.client.event.AddressToLocationsHandler;
import com.esri.gwt.client.event.ErrorHandler;
import com.esri.gwt.client.event.LocationToAddressHandler;
import com.esri.gwt.client.geometry.Point;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.core.client.JsArrayString;

/**
 * Represents a geocode service resource exposed by the ArcGIS Server REST API. It is used to generate candidates 
 * for an address. It also used to find an address for a given location. 
 * 
 * @author Justin Merz
 */
public class Locator extends JavaScriptObject {
	
	protected Locator() {}
	
	/**
	 * Creates a new Locator object.
	 * 
	 * @param url - URL to the ArcGIS Server REST resource that represents a locator service. An example is 
	 * http://sampleserver1.arcgisonline.com/ArcGIS/rest/services/Locators/ESRI_Geocode_USA/GeocodeServer.
	 * @return Locator
	 */
	public static native Locator create(String url) /*-{
		return new $wnd.esri.tasks.Locator(url);
	}-*/;
	
	/**
	 * URL to the ArcGIS Server REST resource that represents a locator service
	 * 
	 * @return String
	 */
	public native final String getUrl() /*-{
		return this.url;
	}-*/;
	
	/**
	 * Sends a request to the ArcGIS REST geocode resource to find candidates for a single address specified in the address 
	 * argument. On completion, the onAddressToLocationsComplete event is fired and the optional callback function is invoked.
	 * 
	 * @param address - The address argument is data object that contains properties representing the various address fields 
	 * accepted by the corresponding geocode service.
	 * @param outFields - The list of fields included in the returned result set. This list is a comma delimited list of field names. 
	 * If you specify the shape field in the list of return fields, it is ignored. For non-intersection addresses you can specify the 
	 * candidate fields as defined in the geocode service. For intersection addresses you can specify the intersection candidate fields.
	 * @param callback - The function to call when the method has completed or if an error object is returned if an error occurs 
	 * on the Server during task execution
	 * @return Deferred
	 */
	public native final Deferred addressToLocations(Address address, JsArrayString outFields, AddressToLocationsCallback callback) /*-{
		this.addressToLocations(address, outFields,
			function(candidates) {
				callback.@com.esri.gwt.client.callback.AddressToLocationsCallback::onAddressToLocationsComplete(Lcom/google/gwt/core/client/JsArray;)(candidates);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.AddressToLocationsCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Sends a request to the ArcGIS REST geocode resource to find candidates for a single address specified in the address 
	 * argument. On completion, the onAddressToLocationsComplete event is fired and the optional callback function is invoked.
	 * 
	 * @param address - The address argument is data object that contains properties representing the various address fields 
	 * accepted by the corresponding geocode service.
	 * @param outFields - The list of fields included in the returned result set. This list is a comma delimited list of field names. 
	 * If you specify the shape field in the list of return fields, it is ignored. For non-intersection addresses you can specify the 
	 * candidate fields as defined in the geocode service. For intersection addresses you can specify the intersection candidate fields.
	 * @return Deferred
	 */
	public native final Deferred addressToLocations(Address address, OutFields outFields) /*-{
		this.addressToLocations(address, outFields);
	}-*/;
	
	/**
	 * Locates an address based on a given point.
	 * 
	 * @param location - The point at which to search for the closest address. The location should be in the same spatial 
	 * reference as that of the geocode service.
	 * @param distance - The distance in meters from the given location within which a matching address should be 
	 * searched. If this parameter is not provided or an invalid value is provided, a default value of 0 meters is used.
	 * @param callback - The function to call when the method has completed or if an error object is returned if an error 
	 * occurs on the Server during task execution.
	 * @return Deferred
	 */
	public native final Deferred locationToAddress(Point location, float distance, LocationToAddressCallback callback) /*-{
		this.locationToAddress(location, distance,
			function(candidate) {
				callback.@com.esri.gwt.client.callback.LocationToAddressCallback::onLocationToAddressComplete(Lcom/esri/gwt/client/tasks/AddressCandidate;)(candidate);
			},
			function(error) {
				callback.@com.esri.gwt.client.callback.LocationToAddressCallback::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Locates an address based on a given point.
	 * 
	 * @param location - The point at which to search for the closest address. The location should be in the same spatial 
	 * reference as that of the geocode service.
	 * @param distance - The distance in meters from the given location within which a matching address should be 
	 * searched. If this parameter is not provided or an invalid value is provided, a default value of 0 meters is used.
	 * @return Deferred
	 */
	public native final Deferred locationToAddress(Point location, float distance) /*-{
		this.addressToLocations(location, distance, null, null);
	}-*/;
	
	/**
	 * Add event handler for addressToLocations
	 * 
	 * @param handler - Fires when Locator.addressToLocation() has completed. 
	 */
	public final native void addAddressToLocationsHandler(AddressToLocationsHandler handler) /*-{
		$wnd.dojo.connect(this, "onAddressToLocationsComplete",
			function(addressCandidates) {
				handler.@com.esri.gwt.client.event.AddressToLocationsHandler::onAddressToLocationsComplete(Lcom/google/gwt/core/client/JsArray;)(addressCandidates);
			}
		);
	}-*/;
	
	/**
	 * Add event handler for errors
	 * 
	 * @param handler - Fires when an error occurs when executing the task.
	 */
	public final native void addErrorHandler(ErrorHandler handler) /*-{
		$wnd.dojo.connect(this, "onError",
			function(error) {
				handler.@com.esri.gwt.client.event.ErrorHandler::onError(Lcom/esri/gwt/client/Error;)(error);
			}
		);
	}-*/;
	
	/**
	 * Add event handler for locationToAddress
	 * 
	 * @param handler - Fires when Locator.locationToAddress() has completed.
	 */
	public final native void addLocationToAddressHandler(LocationToAddressHandler handler) /*-{
		$wnd.dojo.connect(this, "onLocationToAddressComplete",
			function(addressCandidate) {
				handler.@com.esri.gwt.client.event.LocationToAddressHandler::onLocationToAddressComplete(Lcom/esri/gwt/client/tasks/AddressCandidate;)(addressCandidate);
			}
		);
	}-*/;
	
	/**
	 * Class for adding sending outFields in addressToLocations()
	 */
	public static class OutFields extends JavaScriptObject {
		
		protected OutFields() {}
		
		/**
		 * Creates new OutField array
		 * 
		 * @return OutFields
		 */
		public static OutFields create() {
			return JavaScriptObject.createArray().cast();
		}
		
		/**
		 * Add field to array.
		 * 
		 * @param field - field to be added
		 */
		public final native void addField(String field) /*-{
			this.push(field);
		}-*/;
		
	}

}
